@@ -30,56 +30,33 @@ For news, bugfixes, etc. visit the home page for this implementation at
     http://www.wayforward.net/spf/
 """
 
-# CVS Commits since last release (2.0.8):
+# CVS Commits since last release (2.0.9):
 # $Log: spf.py,v $
-# Revision 1.108.2.122  2014/04/29 22:56:48  customdesigned
-# Release 2.0.9
+# Revision 1.108.2.128  2014/09/02 17:31:53  customdesigned
+# Release 2.0.10
 #
-# Revision 1.108.2.121  2014/04/28 21:57:08  customdesigned
-# Ignore void lookups for explanation and type 99 lookup.
+# Revision 1.108.2.127  2014/09/01 21:17:13  kitterma
+# Fix TempError handling of errors from the DNS module.
 #
-# Revision 1.108.2.120  2014/04/24 23:02:15  kitterma
-# Remove redundant check of self.void_lookups.
+# Revision 1.108.2.126  2014/08/02 18:35:50  customdesigned
+# '~' is also an unreserved char in rfc7208.
 #
-# Revision 1.108.2.119  2014/04/22 23:03:42  kitterma
-# Update CHANGELOG to prepare for release.
+# Revision 1.108.2.125  2014/08/02 04:36:48  kitterma
+#   * Fix bug in SPF record parsing that caused all 'whitespace' characters to
+#     be considered valid term separators and not just spaces
 #
-# Revision 1.108.2.118  2014/04/22 22:03:13  kitterma
-# Add processing for new void lookups processing limit.
-#
-# Revision 1.108.2.117  2014/04/22 20:54:42  kitterma
-# Adjust documentation of lookup limits to include RFC 7208
-# Add constants and variables for new void lookup limit
-#
-# Revision 1.108.2.116  2014/04/22 17:10:54  kitterma
-# Default SPF process timeout limit to 20 seconds per RFC 7208 4.6.4.
-#
-# Revision 1.108.2.115  2014/04/22 17:02:55  kitterma
-# Change default DNS timeout to 20 seconds in DNSLookup to better match RFC
-# 7208 4.6.4.
-#
-# Revision 1.108.2.114  2014/04/22 04:56:38  kitterma
-# Add permerror to permitted mx-limit results for rfc4408 to fudge changes for
-# RFC 7208.
-#
-# Revision 1.108.2.113  2014/04/22 04:46:58  kitterma
-# Make mx > 10 a permerror per RFC 7208 and mx-limit test.
-#
-# Revision 1.108.2.112  2014/01/20 22:16:38  customdesigned
-# Rename local var hiding str.
-#
-# Revision 1.108.2.111  2014/01/20 22:03:08  customdesigned
-# Test case and fix for more thorough macro syntax error detection.
-#
-# Revision 1.108.2.110  2013/07/25 21:21:49  kitterma
-# Archive previous commit messages for spf.py in pyspf_changelog.txt and bump version to 2.0.9 for start of follow on work.
+# Revision 1.108.2.124  2014/08/02 04:32:36  kitterma
+# Archive previous commit messages for spf.py in pyspf_changelog.txt and bumpi
+# version to 2.0.10 for start of follow on work.
 #
+# Revision 1.108.2.123  2014/07/30 18:41:18  customdesigned
+# Fix flagging AAAA records in dns_a.  Add --strict option to CLI
 #
 # See pyspf_changelog.txt for earlier CVS commits.
 
 __author__ = "Terence Way, Stuart Gathman, Scott Kitterman"
 __email__ = "pyspf@openspf.org"
-__version__ = "2.0.9: Apr 29, 2014"
+__version__ = "2.0.10: Sep 2, 2014"
 MODULE = 'spf'
 
 USAGE = """To check an incoming mail request:
@@ -153,6 +130,8 @@ def DNSLookup(name, qtype, strict=True, timeout=20):
                 for a in resp.answers] \
              + [((a['name'], a['typename']), a['data'])
                 for a in resp.additional]
+    except AttributeError as x:
+        raise TempError('DNS ' + str(x))
     except IOError as x:
         raise TempError('DNS ' + str(x))
     except DNS.DNSError as x:
@@ -265,12 +244,12 @@ class TempError(Exception):
     "Temporary SPF error"
     def __init__(self, msg, mech=None, ext=None):
         Exception.__init__(self, msg, mech)
-        self.msg = msg
+        self.msg = str(msg)
         self.mech = mech
         self.ext = ext
     def __str__(self):
         if self.mech:
-            return '%s: %s '%(self.msg, self.mech)
+            return '%s: %s'%(self.msg, self.mech)
         return self.msg
 
 class PermError(Exception):
@@ -795,8 +774,10 @@ class query(object):
         if not spf:
             return ('none', 250, EXPLANATIONS['none'])
 
-        # split string by whitespace, drop the 'v=spf1'
-        spf = spf.split()
+        # Split string by space, drop the 'v=spf1'.  Split by all whitespace
+        # casuses things like carriage returns being treated as valid space
+        # separators, so split() is not sufficient.
+        spf = spf.split(' ')
         # Catch case where SPF record has no spaces.
         # Can never happen with conforming dns_spf(), however
         # in the future we might want to give warnings
@@ -1094,9 +1075,8 @@ class query(object):
                         raise PermError('Unknown Macro Encountered', macro) 
                     e = expand_one(expansion, macro[3:-1], JOINERS.get(letter))
                     if letter != macro[2]:
-                        e = urllibparse.quote(e)
+                        e = urllibparse.quote(e,'~')
                     result += e
-
             end = i.end()
         result += s[end:]
         if stripdot and result.endswith('.'):
@@ -1206,14 +1186,10 @@ class query(object):
         """Get a list of IP addresses for a domainname.
         """
         if not domainname: return []
-        if self.strict > 1:
-            alist = self.dns(domainname, A)
-            if len(alist) == 0:
-                raise AmbiguityWarning(
-                        'No %s records found for'%A, domainname)
-            else:
-                return alist
         r = self.dns(domainname, A)
+        if self.strict > 1 and len(r) == 0:
+            raise AmbiguityWarning(
+                    'No %s records found for'%A, domainname)
         if A == 'AAAA' and bytes is str:
           # work around pydns inconsistency plus python2 bytes/str ambiguity
           return [ipaddress.Bytes(ip) for ip in r]
@@ -1888,15 +1864,19 @@ DNS.DiscoverNameServers() # Fails on Mac OS X? Add domain to /etc/resolv.conf
 if __name__ == '__main__':
     import getopt
     try:
-       opts,argv = getopt.getopt(sys.argv[1:],"hv",["help","verbose"])
+       opts,argv = getopt.getopt(sys.argv[1:],"hvs:",
+        ["help","verbose","strict"])
     except getopt.GetoptError as err:
        print(str(err))
        print(USAGE)
        sys.exit(2)
     verbose = False
+    strict = True
     for o,a in opts:
         if o in ('-v','--verbose'):
            verbose = True
+        if o in ('-s','--strict'):
+           strict = int(a)
         elif o in ('-h','--help'):
            print(USAGE)
     if len(argv) == 0:
@@ -1913,7 +1893,8 @@ if __name__ == '__main__':
             print("PermError: ", x)
     elif len(argv) == 3:
         i, s, h = argv
-        q = query(i=i, s=s, h=h,receiver=socket.gethostname(),verbose=verbose)
+        q = query(i=i, s=s, h=h,receiver=socket.gethostname(),verbose=verbose,
+                strict=strict)
         print(q.check(),q.mechanism)
         if q.perm_error and q.perm_error.ext:
             print(q.perm_error.ext)
